locals {
  mdr_lib_version_hourly  = lookup(var.job_config_hourly, "mdr_lib_version", var.default_mdr_lib_version)
  mdr_lib_version_24hours = lookup(var.job_config_24hours, "mdr_lib_version", var.default_mdr_lib_version)
  mdr_lib_version_daily   = lookup(var.job_config_daily, "mdr_lib_version", var.default_mdr_lib_version)
  mdr_lib_version_31days  = lookup(var.job_config_31days, "mdr_lib_version", var.default_mdr_lib_version)
  glue_handler_prefix     = coalesce(var.glue_handler_prefix, var.aggregation_name)
  aggregation_name        = replace(var.aggregation_name, "-", "_")
}

resource "aws_glue_job" "glue_hourly_job" {
  name                   = "${var.name_prefix}-${var.aggregation_name}-hourly"
  tags                   = merge(var.tags, { aggregation_type = var.aggregation_name, aggregation_window = "hourly" })
  depends_on             = [aws_glue_workflow.daily]
  glue_version           = "3.0"
  role_arn               = var.job_role_arn
  max_retries            = 0
  timeout                = var.job_config_hourly.timeout
  number_of_workers      = var.job_config_hourly.num_workers
  worker_type            = var.job_config_hourly.worker_type
  security_configuration = var.security_configuration_name

  default_arguments = {
    # Caution -- this requires this to align with the table definition
    "--dst_table_name" = length(var.combo_job_map) == 0 ? "${var.database_name}.${local.aggregation_name}_hourly" : "\"\""
    "--src_table_name" = var.src_tbl_name
    # This indicates that this is a x-account job
    "--src_account_id"                   = var.src_account_id
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.tools_bucket_name}/lib/mdr_prev-${local.mdr_lib_version_hourly}-py3-none-any.whl"
    "--extra-files"                      = "s3://${var.tools_bucket_name}/resources/log4j.properties"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "botocore==1.25.2,boto3==1.22.1,wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/${var.aggregation_name}/${local.aggregation_name}_hourly/"
    "--env"                              = var.env
    "--athena_workgroup"                 = var.athena_workgroup_name
    "--downstream_workflow_name"         = aws_glue_workflow.daily.name
    "--wavefront_url"                    = var.wavefront_url
    "--mcs2_env"                         = var.mcs2_env
    # These are needed only for combo jobs
    "--table_name_partition_id_mapper" = jsonencode([
      for ele in var.combo_job_map :
      { dst_table_name = "${var.database_name}.${ele.agg_name}_hourly", partition_id_col = ele.partition_id_col }
    ])
    "--partition_id_str_len" = try(var.job_config_hourly.partition_id_str_len, "\"\"")
  }

  command {
    script_location = "s3://${var.tools_bucket_name}/jobs/${local.glue_handler_prefix}/glue_handler_hourly-${local.mdr_lib_version_hourly}.py"
  }

  execution_property {
    max_concurrent_runs = var.job_config_hourly["max_concurrency"]
  }

  # This would be useful, but I suspect in mcs2 we'll have to manage via helm.
  # Updates could be done in console for hot-fixes, but would need to be synced with helm.
  # lifecycle {
  #   ignore_changes = [
  #     number_of_workers,
  #     timeout,
  #     worker_type,
  #     execution_property[0].max_concurrent_runs
  #   ]
  # }
}

resource "aws_glue_job" "glue_24hours_job" {
  name                   = "${var.name_prefix}-${var.aggregation_name}-24hours"
  tags                   = merge(var.tags, { aggregation_type = var.aggregation_name, aggregation_window = "24hours" })
  glue_version           = "3.0"
  role_arn               = var.job_role_arn
  max_retries            = 0
  timeout                = var.job_config_24hours.timeout
  number_of_workers      = var.job_config_24hours.num_workers
  worker_type            = var.job_config_24hours.worker_type
  security_configuration = var.security_configuration_name

  default_arguments = {
    # Caution -- this requires this to align with the table definition
    "--dst_table_name"                   = length(var.combo_job_map) == 0 ? "${var.database_name}.${local.aggregation_name}_24hours" : "\"\""
    "--src_table_name"                   = "${var.database_name}.${local.aggregation_name}_hourly"
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.tools_bucket_name}/lib/mdr_prev-${local.mdr_lib_version_24hours}-py3-none-any.whl"
    "--extra-files"                      = "s3://${var.tools_bucket_name}/resources/log4j.properties"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/${var.aggregation_name}/${local.aggregation_name}_24hours/"
    "--env"                              = var.env
    "--athena_workgroup"                 = var.athena_workgroup_name
    "--wavefront_url"                    = var.wavefront_url
    "--mcs2_env"                         = var.mcs2_env

    # These are only needed for some jobs
    "--num_src_partition_characters" = try(var.job_config_24hours.num_src_partition_characters, "")
    "--num_output_groups"            = try(var.job_config_24hours.num_output_groups, "")
    "--num_parallel_jobs"            = try(var.job_config_24hours.num_parallel_jobs, "")

    # These are needed only for combo jobs
    "--table_name_partition_id_mapper" = jsonencode([
      for ele in var.combo_job_map :
      { dst_table_name = "${var.database_name}.${ele.agg_name}_24hours", partition_id_col = ele.partition_id_col }
    ])
    "--partition_id_str_len" = try(var.job_config_24hours.partition_id_str_len, "\"\"")
  }

  command {
    script_location = "s3://${var.tools_bucket_name}/jobs/${local.glue_handler_prefix}/glue_handler_24hours-${local.mdr_lib_version_24hours}.py"
  }

  execution_property {
    max_concurrent_runs = var.job_config_24hours["max_concurrency"]
  }

  # This would be useful, but I suspect in mcs2 we'll have to manage via helm.
  # Updates could be done in console for hot-fixes, but would need to be synced with helm.
  # lifecycle {
  #   ignore_changes = [
  #     number_of_workers,
  #     timeout,
  #     worker_type,
  #     execution_property[0].max_concurrent_runs
  #   ]
  # }
}

resource "aws_glue_job" "glue_daily_job" {
  name                   = "${var.name_prefix}-${var.aggregation_name}-daily"
  tags                   = merge(var.tags, { aggregation_type = var.aggregation_name, aggregation_window = "daily" })
  glue_version           = "3.0"
  role_arn               = var.job_role_arn
  max_retries            = 0
  timeout                = var.job_config_daily.timeout
  number_of_workers      = var.job_config_daily.num_workers
  worker_type            = var.job_config_daily.worker_type
  security_configuration = var.security_configuration_name

  default_arguments = {
    # Caution -- this requires this to align with the table definition
    "--dst_table_name"                   = length(var.combo_job_map) == 0 ? "${var.database_name}.${local.aggregation_name}_daily" : "\"\""
    "--src_table_name"                   = "${var.database_name}.${local.aggregation_name}_hourly"
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.tools_bucket_name}/lib/mdr_prev-${local.mdr_lib_version_daily}-py3-none-any.whl"
    "--extra-files"                      = "s3://${var.tools_bucket_name}/resources/log4j.properties"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/${var.aggregation_name}/${local.aggregation_name}_daily/"
    "--env"                              = var.env
    "--athena_workgroup"                 = var.athena_workgroup_name
    "--wavefront_url"                    = var.wavefront_url
    "--mcs2_env"                         = var.mcs2_env

    # These are only needed for some jobs
    "--num_src_partition_characters" = try(var.job_config_daily.num_src_partition_characters, "")
    "--num_output_groups"            = try(var.job_config_daily.num_output_groups, "")
    "--num_parallel_jobs"            = try(var.job_config_daily.num_parallel_jobs, "")
    # These are needed only for combo jobs
    "--table_name_partition_id_mapper" = jsonencode([
      for ele in var.combo_job_map :
      { dst_table_name = "${var.database_name}.${ele.agg_name}_daily", partition_id_col = ele.partition_id_col }
    ])
    "--partition_id_str_len" = try(var.job_config_daily.partition_id_str_len, "\"\"")
  }

  command {
    script_location = "s3://${var.tools_bucket_name}/jobs/${local.glue_handler_prefix}/glue_handler_daily-${local.mdr_lib_version_daily}.py"
  }

  execution_property {
    max_concurrent_runs = var.job_config_daily["max_concurrency"]
  }

  # This would be useful, but I suspect in mcs2 we'll have to manage via helm.
  # Updates could be done in console for hot-fixes, but would need to be synced with helm.
  # lifecycle {
  #   ignore_changes = [
  #     number_of_workers,
  #     timeout,
  #     worker_type,
  #     execution_property[0].max_concurrent_runs
  #   ]
  # }
}

resource "aws_glue_job" "glue_31days_job" {
  name                   = "${var.name_prefix}-${var.aggregation_name}-31days"
  tags                   = merge(var.tags, { aggregation_type = var.aggregation_name, aggregation_window = "31days" })
  glue_version           = "3.0"
  role_arn               = var.job_role_arn
  max_retries            = 0
  timeout                = var.job_config_31days.timeout
  number_of_workers      = var.job_config_31days.num_workers
  worker_type            = var.job_config_31days.worker_type
  security_configuration = var.security_configuration_name

  default_arguments = {
    # Caution -- this requires this to align with the table definition
    "--dst_table_name"                   = length(var.combo_job_map) == 0 ? "${var.database_name}.${local.aggregation_name}_31days" : "\"\""
    "--src_table_name"                   = "${var.database_name}.${local.aggregation_name}_daily"
    "--TempDir"                          = "s3://${var.processing_bucket_name}/tmp/"
    "--extra-py-files"                   = "s3://${var.tools_bucket_name}/lib/mdr_prev-${local.mdr_lib_version_31days}-py3-none-any.whl"
    "--extra-files"                      = "s3://${var.tools_bucket_name}/resources/log4j.properties"
    "--enable-continuous-cloudwatch-log" = "true"
    "--enable-glue-datacatalog"          = ""
    "--enable-metrics"                   = ""
    "--additional-python-modules"        = "wavefront-sdk-python"
    "--enable-spark-ui"                  = "true"
    "--spark-event-logs-path"            = "s3://${var.processing_bucket_name}/logs/${var.aggregation_name}/${local.aggregation_name}_31days/"
    "--env"                              = var.env
    "--athena_workgroup"                 = var.athena_workgroup_name
    "--wavefront_url"                    = var.wavefront_url
    "--mcs2_env"                         = var.mcs2_env

    # These are only needed for some jobs
    "--num_src_partition_characters" = try(var.job_config_31days.num_src_partition_characters, "")
    "--num_output_groups"            = try(var.job_config_31days.num_output_groups, "")
    "--num_parallel_jobs"            = try(var.job_config_31days.num_parallel_jobs, "")
    # These are needed only for combo jobs
    "--table_name_partition_id_mapper" = jsonencode([
      for ele in var.combo_job_map :
      { dst_table_name = "${var.database_name}.${ele.agg_name}_31days", partition_id_col = ele.partition_id_col }
    ])
    "--partition_id_str_len" = try(var.job_config_31days.partition_id_str_len, "\"\"")
  }

  command {
    script_location = "s3://${var.tools_bucket_name}/jobs/${local.glue_handler_prefix}/glue_handler_31days-${local.mdr_lib_version_31days}.py"
  }

  execution_property {
    max_concurrent_runs = var.job_config_daily["max_concurrency"]
  }

  # This would be useful, but I suspect in mcs2 we'll have to manage via helm.
  # Updates could be done in console for hot-fixes, but would need to be synced with helm.
  # lifecycle {
  #   ignore_changes = [
  #     number_of_workers,
  #     timeout,
  #     worker_type,
  #     execution_property[0].max_concurrent_runs
  #   ]
  # }
}
